==> enterprise-0.2.0-20200607.221140-115.jar: degree9/es6.clj <==
(ns degree9.es6
  (:refer-clojure :exclude [class])
  (:require [cljs.analyzer :as ana]
            [cljs.compiler :as compiler]))

;; Native ES6 Class compiler extension ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(alter-var-root #'ana/specials #(conj % 'class* 'class-method* 'class-ctor*))

(defmethod ana/parse 'class*
  [op env [_ sym & exprs :as form] _ _]
  (let [extends    (and (= (first exprs) :extends) (second exprs))
        implements (and (= (first exprs) :implements) (second exprs))
        exprs      (if (or extends implements) (drop 2 exprs) exprs)]
    {:env env
     :op :class
     :children [:methods]
     :methods (ana/disallowing-recur
                 (->> (rest exprs)
                      (mapv #(ana/analyze (assoc env :context :method) %))))
     :ctor (first exprs)
     :extends extends
     :implements implements
     :form form
     :class sym}))

(defmethod compiler/emit* :class
  [{:keys [ctor methods class]}]
  (cond extends    (compiler/emitln "class " class " extends " extends "{")
        implements (compiler/emitln "class " class " implements " implements "{")
        :else      (compiler/emitln "class " class "{"))
  (compiler/emitln "constructor( " (interpose "," (map compiler/munge (second ctor))) ") {")
  (compiler/emits "}")
  (doseq [m methods]
    (compiler/emitln m))
  (compiler/emits "}"))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Public Async/Await API ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro class
  "Wraps body in native es6 class block."
  [name & body]
  `(~'class* ~name ~@body))

(defmacro defclass
  "Define a native es6 class."
  [name & body]
  `(def ~name (class ~name ~@body)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

==> enterprise-0.2.0-20200607.221447-116.jar: degree9/es6.clj <==
(ns degree9.es6
  (:refer-clojure :exclude [class])
  (:require [cljs.analyzer :as ana]
            [cljs.compiler :as compiler]))

;; Native ES6 Class compiler extension ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(alter-var-root #'ana/specials #(conj % 'class* 'class-method* 'class-ctor*))

(defmethod ana/parse 'class*
  [op env [_ sym & exprs :as form] _ _]
  (let [extends    (and (= (first exprs) :extends) (second exprs))
        implements (and (= (first exprs) :implements) (second exprs))
        exprs      (if (or extends implements) (drop 2 exprs) exprs)]
    {:env env
     :op :class
     :children [:methods]
     :methods (ana/disallowing-recur
                 (->> (rest exprs)
                      (mapv #(ana/analyze (assoc env :context :method) %))))
     :ctor (first exprs)
     :extends extends
     :implements implements
     :form form
     :class sym}))

(defmethod compiler/emit* :class
  [{:keys [ctor methods class]}]
  (compiler/emit "class " class)
  (when extends (compiler/emit " extends " extends))
  (when implements (compiler/emit " implements " implements))
  (compiler/emitln " {")
  (compiler/emitln "constructor( " (interpose "," (map compiler/munge (second ctor))) ") {")
  (compiler/emits "}")
  (doseq [m methods]
    (compiler/emitln m))
  (compiler/emits "}"))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Public Async/Await API ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro class
  "Wraps body in native es6 class block."
  [name & body]
  `(~'class* ~name ~@body))

(defmacro defclass
  "Define a native es6 class."
  [name & body]
  `(def ~name (class ~name ~@body)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
